@@ -211,10 +211,8 @@ int main(int argc, char* argv[])
     text_create();
     ui_create();
     viewer_create();
+
     config_init();
-    font_init();
-    info_init();
-    viewer_init();
 
     // parse command arguments
     argn = parse_cmdargs(argc, argv);
@@ -223,6 +221,12 @@ int main(int argc, char* argv[])
         goto done;
     }
 
+    font_init();
+    info_init();
+    viewer_init();
+
+    config_destroy(); // configuration is applied and can be freed
+
     // compose image list
     if (image_list_init((const char**)&argv[argn], argc - argn) == 0) {
         fprintf(stderr, "No images to view, exit\n");
@@ -254,14 +258,13 @@ int main(int argc, char* argv[])
     rc = ui_run();
 
 done:
-    config_free();
-    viewer_free();
-    ui_free();
-    image_list_free();
-    loader_free();
-    info_free();
-    font_free();
-    keybind_free();
+    viewer_destroy();
+    loader_destroy();
+    ui_destroy();
+    image_list_destroy();
+    info_destroy();
+    font_destroy();
+    keybind_destroy();
 
     return rc ? EXIT_SUCCESS : EXIT_FAILURE;
 }
